//+------------------------------------------------------------------+
//|                                          LuKu_DONNYSOK_EA_v2.mq4 |
//|                                                             LuKu |
//|                                               http://luku.cz/fx/ |
//+------------------------------------------------------------------+
#property copyright "LuKu"
#property link      "http://luku.cz/fx/"
/*

  EAcko pro vstup a vystup dle rozkmitane PL krivky, inspirace podle LuKu_SIDSOK_ENTRY_v4.mq4
  Manualni vstupni skript na http://luku.cz/fx/2011/08/10/sidsok-entry-v4/
  Diskuze: http://www.fxstreet.cz/diskusni-forum+statisticka-arbitrage.html (od strany cca 90)
   
  Verze 1 (11.8.2011)
   - zakladni naprogramovani
   - pokud je pozadovan prvni vstupni obchod obracene, musi se zatim otocit pary
   - promenna pro otoceni obchodu se uklada do globalnich promennych klavesa F3 v MT4, pokud je pritomna pozice to pro dalsi obchod otaci (samozrejme pouze v pripade OpenAgainChangeDirection = true)
   - pripadny restart strategie provadime klavsou F7 a OK
  Verze 2 (17.8.2011)
   - EA prejmenovano na DONNYSOK :)
   - zmena na asynchronni zpracovavani:
      - EAcko umoznuje pridat do vsech paru ktere obchodujeme
      - zjistime se tak veskere ticky ktere obchod ovlivnuji a maximalizujeme rychlost uzavirani
      - spustene EAcka si zajistuji komunikaci mezi sebou a klicovou akci vzdy dela pouze jedno konkretni EAcko
      - dalsi moznosti je pouzit pro ostatni pary http://luku.cz/fx/wp-content/uploads/LuKu_TickEmulator_v10.mq4 ktery zasila ticky do naseho okna s EA
   - upravena preference velikosti lotu pri uzavirani + uzavirani predelano na bezpecnejsi uzavirani pres tickety
   - doplnen error managment, chyby ktere lze opravit se snazi EAcko opravit (requote, busy)
   - do zprav dodany dalsi informace (o dokonceni otevirani / uzavirani, pri uzavirani vyse zisku)
   - zmenneno vychozi nastaveni a nove se po ukonceni obchodu neotaci smer obchodu
   - nove se do profitu zapocitava i swap a komise
   
*/

string ShortName = "DONNYSOKEAV2";

extern string Buy         = "Symboly ke koupi oddelene carkou";
extern string Buys_Long   = "GBPUSD,EURJPY"; //donny pary

extern string Sell        = "Symboly k prodeji oddelene carkou";
extern string Sells_Short = "EURUSD,GBPJPY"; // donny pary

extern string SLotSizes = "Velikosti lotu oddelenych carkou";
extern string LotSizes = "0.03,0.06,0.1,0.13,0.16,0.2,0.23,0.26";

extern string SLotRanges = "Velikosti pipu vzdalenosti oddelenych carkou";
extern string LotRanges = "10,20,40,80,160,320,640,1280";

extern string sCloseProfit = "Pri jakem profitu obchod uzavrit?";
extern double CloseProfit = 10.0;

extern string sOpenAgain = "Po uzavreni otevrit automaticky znovu?";
extern bool   OpenAgain = true;
extern string sOpenAgainSameDirection = "Po znovu otevreni zmenit smer cekacek?";
extern bool   OpenAgainChangeDirection = false;

extern int MagickNumber = 1024; // magick number pouzivany pro obchod
extern int MaxSlippage = 5; // maximalni hodnota prokluzu
extern bool SendEmailAboutProfit = false; // zaslat email o uzavreni s profitem?

// win veci
#include <stderror.mqh>
#include <stdlib.mqh>

// interni promenne
string aBuyPairs[];  // pole naparsovanych paru nakup
string aSellPairs[]; // pole naparsovanych paru prodej
string aLotSizes[];  // pole lotu
string aLotRanges[]; // pole rozsahu pro cekacky
string m; // parametr mini uctu napriklad IBFX
string Res;
bool init; // povedl se init?
int Err;
bool StopTrade; // nouzove zastaveni programem
int internalMN = 0; // MN pro lokovani
string lockName; // promenna s nazvem pro lokovani
double dProfit = 0.0; // profit pro uzavirani

//+------------------------------------------------------------------+
//| expert initialization function                                   |
//+------------------------------------------------------------------+
int init(){
   init = false; // zatim to neprobehlo
   StopTrade = false; // nulujeme stopnute obchodovani
   
   Comment("Init, cekam na TICK ...");
   
   // mazeme globalni promenne o close (aby slo udelat jednorazovy obchod znovu)
   GlobalVariablesDeleteAll(StringConcatenate(ShortName,"_CLOSED_"));
   
   // uprava uzivatelskeho vstupu
   Buys_Long = StringUpperCase(StringTrimLeft(StringTrimRight(Buys_Long)));
   Sells_Short = StringUpperCase(StringTrimLeft(StringTrimRight(Sells_Short)));

   // zjisteni typu uctu - parametr ktery se pridava k parum
   m = StringSubstr(Symbol(),6,StringLen(Symbol())-6);
   
   // parsovani symbolu
   stringToArray(aBuyPairs, Buys_Long);
   stringToArray(aSellPairs, Sells_Short);
   
   // existuji zadane pary?
   Res = testPairsIfExists(aBuyPairs);
   if(Res != ""){
     Alert(StringConcatenate("Neexistujici Buy par! - '",Res,"'"));
     return(false);
   }
   // existuji zadane pary?
   Res = testPairsIfExists(aSellPairs);
   if(Res != ""){
     Alert(StringConcatenate("Neexistujici Sell par! - '",Res,"'"));
     return(false);
   }
   
   // parsovani lotu
   stringToArray(aLotSizes, LotSizes);
   // parsovani rozsahu
   stringToArray(aLotRanges, LotRanges);
   
   // overime pocet lotu VS pocet Rozsahu
   if(ArraySize(aLotSizes) != ArraySize(aLotRanges)){
     Alert("Nesedi pocet hodnot LotSizes vs LotRanges!");
     return(false);
   }
   
   // vytvorime nase unikatni MN
   internalMN = makeMagicNumber(Symbol());
   // nazev promenne pro lokovani
   lockName = StringConcatenate(ShortName,"_LOCKED");
   unlock();
   
   init = true; // vse ok!
   
   return(0);
}

//+------------------------------------------------------------------+
//| expert deinitialization function                                 |
//+------------------------------------------------------------------+
int deinit(){
   unlock();
   return(0);
}

//+------------------------------------------------------------------+
//| expert start function                                            |
//+------------------------------------------------------------------+
int start(){
   // byl init ok?
   if(init == false){
     Comment("Init se nezdaril, neobchodujeme!");
     return(0);
   }
   // mame povoleno obchodovani?
   if(IsConnected() != true || IsTradeAllowed() != true){
     Comment("Obchodovani neni povoleno nebo nemame spojeni!");
     return(0);
   }
   // nemame zastavene obchodovani?
   if(StopTrade == true){
     Comment("Obchodovani POZASTAVENO systemem.");
     return(0);
   }
   
   // zamykani
   if(lock() == false){
     // nemame lock
     Comment("Zamek ma jiny proces!");
     return(0);
   }
   
   double dDir; // smer obchodovani
   
   // testneme zda nemame jiz otevrene obchody
   int iRes = isExistTradeByMagickNumber(MagickNumber);
   if(iRes == -1){
     Comment("Error pri vyhledavani obchodu s MagickNumber!");
     unlock();
     return(0); // chyba zkusime to pozdeji
   }
   if(iRes == 1){ // ano obchod existuje
     // jaky mame zisk?
     double profit = getOrderProfit(MagickNumber);
     if(profit == -999999){
       // chyba
       Comment("Nelze zjistit profit!");
       unlock();
       return(0); // nepodarilo se zjistit profit
     }
     if(CloseProfit >= profit){
       // jeste profit nemame
       Comment(StringConcatenate("Cekame na profit ",DoubleToStr(CloseProfit,2),", zatim ",DoubleToStr(profit,2)));
     }else{
       // uz jsme dosahli profitu, uzavirame!
       Comment("Uzaviram pozice!");
       Alert("Dosazen PROFIT - uzavirame pozice!");
       dProfit = AccountBalance();
       // uzavirame profit dle objemu u otevrenych obchodu
       subCloseTrade(MagickNumber, 2, MaxSlippage, true);
       // a ted vsechny cekacky
       subCloseTrade(MagickNumber, 0, MaxSlippage, false);
       Comment("Pozice uzavreny!");
       dProfit = AccountBalance() - dProfit;
       Alert(StringConcatenate("Skutecny profit: ",DoubleToStr(dProfit, 2),", uzavirany profit: ",DoubleToStr(profit,2)));
       if(SendEmailAboutProfit == true){
         SendMail(StringConcatenate(TimeToStr(TimeCurrent(), TIME_MINUTES)," ",ShortName+" Profit: ",DoubleToStr(dProfit, 2)),"");
       }
       // nastavime si promennou
       GlobalVariableSet(StringConcatenate(ShortName,"_CLOSED_",DoubleToStr(MagickNumber,0)),1);
       // otaceni promennych kvuli otaceni obchodu
       if(OpenAgainChangeDirection == true){
         // ziskani predchozi promenne a jeji otoceni
         dDir = GlobalVariableGet(StringConcatenate(ShortName,"_DIRECTION_",DoubleToStr(MagickNumber,0)));
         if(dDir == 0){
           dDir = 1;
         }else{
           dDir = 0;
         }
       }else{
         dDir = 0; // nikdy neotacet vzdy v standardnim smeru
       }
       // ulozeni smerove promenne
       GlobalVariableSet(StringConcatenate(ShortName,"_DIRECTION_",DoubleToStr(MagickNumber,0)), dDir);
       // ulozeni informace o uzavreni obchodu
       GlobalVariableSet(StringConcatenate(ShortName,"_CLOSED_",DoubleToStr(MagickNumber,0)),1);
       unlock();
       return(0); // vyskocime a dalsim tickem zahajime akci znovu
     }
   }

   // nic neexistuje takze otevirame obchod
   if(iRes == 0){
     // nemame pozadavek na neotevirani dalsich obchodu?
     if(OpenAgain == false){
       // testujeme zda existuje globalni promenna s potvrzenim predchoziho obchodu
       if(GlobalVariableCheck(StringConcatenate(ShortName,"_CLOSED_",DoubleToStr(MagickNumber,0))) == true){
         // neotevirame to dale
         Comment("Obchodovani zastaveno po prvnim uspesnem obchodu - pro dalsi obchod restartujte strategii.");
         unlock();
         return(0);
       }
     }
     Comment("Otevirame nove pozice!");
     Alert("Otevirame nove pozice!");
     // smery
     int bUpType, bDownType, sUpType, sDownType;
     if(GlobalVariableGet(StringConcatenate(ShortName,"_DIRECTION_",DoubleToStr(MagickNumber,0))) == 0){
       // standardni smer
       bUpType = OP_BUYSTOP;
       bDownType = OP_BUYLIMIT;
       sUpType = OP_SELLLIMIT;
       sDownType = OP_SELLSTOP;
     }else{
       // otoceny smer
       bUpType = OP_SELLLIMIT;
       bDownType = OP_SELLSTOP;
       sUpType = OP_BUYSTOP;
       sDownType = OP_BUYLIMIT;
     }
       
     // otevreme to znovu
     if(createTrades(bUpType, bDownType, sUpType, sDownType) == true){
       Comment("Obchody zavedeny!");
       Alert("Obchody zavedeny!");
     }else{
       // neco se nam nepodarilo ihned to zavirame a koncime
       Alert("ERROR pri zrizovani obchodu: "+GetLastError());
       // rusime zavedene pokyny dle magick number
       if(subCloseTrade(MagickNumber) == true){
         Alert("Obchody korektne zruseny, ale obchodovani ZASTAVENO!");
         StopTrade = true; // muze se jednat o nekonecnou smycku proto je nutne to zastavit
       }else{
         Alert("POZOR! obchody nebyly zruseny!");
       }
       // zkusime to za chvilku
       Sleep(10000); // 10 sec
     }
   }
   
   unlock();
   return(0);
}
//+------------------------------------------------------------------+

// preparsovani hodnot oddelenych oddelovacem do pole
// aResult - pole cilovych retezcu
// values - z ceho parsujeme
// delimiter - co nase texty oddeluje
void stringToArray(string &aResult[], string &values, string delimiter = ","){
   // vymazani pole
   ArrayResize(aResult,0);
   int s = 0;
   int i = 0;
   string current;
   // dodani posledni carky kvuli podpore i jednoho 
   if (StringSubstr(values,StringLen(values),1) != delimiter){
     values = StringConcatenate(values, delimiter);
   }
   // prvni vyskyt
   i = StringFind(values, delimiter, s);
   // dokud tam neco je tak parsujeme
   while (i > 0){
     current = StringSubstr(values, s, i-s); // vyparsovani prvni hodnoty
     ArrayResize(aResult,ArraySize(aResult)+1); // pridame dalsi pozici
     aResult[ArraySize(aResult)-1] = current; // pridame dalsi hodnotu
     s = i + 1;
     i = StringFind(values, delimiter, s); // dalsi vyskyt
   }
}

// prevod na velka pismena
string StringUpperCase(string str){
   string   s = str;
   int      lenght = StringLen(str) - 1;
   int      char;
   while(lenght >= 0){
     char = StringGetChar(s, lenght);
     if((char > 96 && char < 123) || (char > 223 && char < 256)){
       s = StringSetChar(s, lenght, char - 32);
     }else {
       if(char > -33 && char < 0){
         s = StringSetChar(s, lenght, char + 224);
       }
     }
     lenght--;
   }
   return(s);
}

// otestuje zda vsechny pary v poli existuji + upravuje pary pro mini ucty, poud je to potreba
// "" vse ok, "PAR" jinak vraci par ktery neexistuje
string testPairsIfExists(string &aPairs[]){
  for(int i=0; i<ArraySize(aPairs); i++){
    if(iClose(aPairs[i],0,0) == 0){
      // zkusime s m
      if(iClose(StringConcatenate(aPairs[i],m),0,0) == 0){
        return(aPairs[i]+m); // neexistuje
      }else{
        // existuje, zaznamename
        aPairs[i] = StringConcatenate(aPairs[i],m);
      }
    }
  }
  return("");
}

// test na existenci obchodu s konkretnim magickNumber
int isExistTradeByMagickNumber(int magick){
  // projedeme vsechny obchody a zkusime najit stejne cislo
  for(int i=OrdersTotal()-1;i>=0;i--){
    if(OrderSelect(i, SELECT_BY_POS) == false){
      // objednavku nelze vybrat
      Sleep(1000);
      return(-1);
    }
    if(magick == OrderMagicNumber() || magick == StrToInteger(OrderComment())){
      // nalezeno
      return(1);
    }
  }
  return(0);
}

// vrati aktualni vysi zisku obchodu dle magickNum a ev dle symbolu
double getOrderProfit(int MN, string symbol = ""){
  double profit = 0; // aktualni profit
  // projedeme vsechny obchody
  for(int i=OrdersTotal()-1;i>=0;i--){
    if(OrderSelect(i, SELECT_BY_POS) == false){
      // objednavku nelze vybrat
      Sleep(1000);
      return(-999999);
    }
    if(MN == OrderMagicNumber() || MN == StrToInteger(OrderComment())){
      // mame pozadavek na filtraci symbolu?
      if(symbol == ""){
        // nalezeno, ziskani profitu
        profit += getProfit();
      }else{
        // filtrace dle symbolu
        if(OrderSymbol() == symbol){
          // mame
          profit += getProfit();
        }
      }
    }
  }
  return(profit);
}

// ziska profit z aktualne zvoleneho profitu
double getProfit(){
  return(OrderProfit() + OrderSwap() + OrderCommission());
}

/*
  Funkce na uzavirani otevrenych obchodu
  magickNum:
    magick number, pokud se nevyplni uzavira vse
  type:
    0  - uzavira vsechny obchody
    1  - uzavira pouze profit obchody
   -1  - uzavira pouze ztratove obchdoy
    2  - uzavira vsechny otevrene obchody mimo cekacich
  byVolume:
    True - uzavira prvne obchody s nejvetsim pocetem lotu
    False - neresi se
  symbol:
    - uzavirani dle symbolu
  Return:
    True - vse se podarilo ok
    False - nastala chyba!
*/
bool subCloseTrade(int magickNum, int type = 0, int MaxSlippage = 5, bool byVolume = false, string symbol = ""){
  int err;
  string errTxt;
  int count, maxCycle = 10; // kolikrat se zkouset uzavirani opravit (pokud je to mozne) nez se vygeneruhe hruba chyba
  int iOrders[], iIndex[];
  double iProfit[];
  int i,iO;

  // mame pozadavek na uzavirani dle volume?
  iO = OrdersTotal();
  ArrayResize(iIndex,iO);  // aktualni indexy
  ArrayResize(iProfit,iO); // profity k indexum
  ArrayResize(iOrders,iO);
  for(i=0;i<iO;i++){
    count = maxCycle;
    while(true){
      if(OrderSelect(i, SELECT_BY_POS) == false){
        // objednavku nelze vybrat
        if(alertCloseDeleteTradeError(GetLastError()) == true){
          // muzeme to jeste zkusit
          if(count == 0){ // uz to zkousime mockrat
            return(false);
          }
          count--; // snizujeme pocet pokusu
        }else{
          return(false); // neopravitelna chyba
        }
      }else{ // vse ok pokracujeme
        break;
      }
    }
    if(byVolume == true){
      iIndex[i] = OrderTicket();
      iProfit[i] = OrderLots();
    }else{
      iOrders[i] = OrderTicket();
    }
  }
  if(byVolume == true){
    // radit dle profitu
    sortArray(iIndex, iProfit, iOrders); // serazeni podle profitu
  }
  
  // uzavirani
  for(i=0; i<ArraySize(iOrders); i++){
    count = maxCycle;
    while(true){
      if(OrderSelect(iOrders[i], SELECT_BY_TICKET) == false){
        // objednavku nelze vybrat
        if(alertCloseDeleteTradeError(GetLastError(), iOrders[i]) == true){
          // muzeme to jeste zkusit
          if(count == 0){ // uz to zkousime mockrat
            return(false);
          }
          count--; // snizujeme pocet pokusu
        }else{
          return(false); // neopravitelna chyba
        }
      }else{ // vse ok pokracujeme v uzavirani
        break;
      }
    }
    // mame stejne magicknumber? pokud ne tak preskakujeme
    if(OrderMagicNumber() != magickNum && StrToInteger(OrderComment()) !=  magickNum) continue;
    if(type == 1){
      // uzavirame pouze ziskove obchody otevrene
      if(getProfit() <= 0) continue;
    }else{
      if(type == -1){
        // uzavirame pouze ztratove
        if(getProfit() >= 0) continue;
      }else{
        // uzavirame vsechny otevrene obchody (ne vsak cekacky)
        if(type == 2){
          if(OrderType() != OP_BUY && OrderType() != OP_SELL){
            continue;
          }
        }
      }
    }
    // nebyl pozadavek na uzivirani pouze konkretniho symbolu?
    if(symbol != "" && symbol != OrderSymbol()){
      // preskakujeme
      continue;
    }
    // uzavirani
    bool result;
    double price;
    
    count = maxCycle;
    bool loop = true;
    while(loop == true){ // ve smycce zejmena kvuli requote, kdy je nutne znovu nastavit cenu
      RefreshRates(); // obnoveni cen
      price = 0.0; // cena pro uzavirani
      switch(OrderType()){
        //Close opened long positions
        case OP_BUY:
          price = MarketInfo(OrderSymbol(), MODE_BID);
        case OP_SELL:
          if(price == 0.0){
            price = MarketInfo(OrderSymbol(), MODE_ASK);
          }
        case OP_BUYLIMIT  :
        case OP_BUYSTOP   :
        case OP_SELLLIMIT :
        case OP_SELLSTOP  :
        default:
          // pokud je cena 0.0 jedna se o limyty jinak zavirame obchod klasicky
          if(price == 0.0){
            // rusime cekacky
            result = OrderDelete(iOrders[i]);
          }else{
            // zavirame obchody
            result = OrderClose(iOrders[i], OrderLots(), price, MaxSlippage, Red);
          }
          if(result == false){
            // nelze to uzavrit
            if(alertCloseDeleteTradeError(GetLastError(), iOrders[i]) == true){
              // muzeme to jeste zkusit
              if(count == 0){ // uz to zkousime mockrat
                return(false);
              }
              count--; // snizujeme pocet pokusu
            }else{
              return(false); // neopravitelna chyba
            }
          }else{ // vse ok pokracujeme v uzavirani
            loop = false; // while break
          }
      }
    }
  }// for
  return(true);
}

/*
  Vypise chybu pri uzavirani obchodu
  Pokud je chyba prechodna vraci True (Staci se o uzavirani pokouset znovu)
  Pokud je chyba konecna / zavazna vraci False
*/ 
bool alertCloseDeleteTradeError(int Error, string info = ""){
  if(Error == 0) return(true);
  string errTxt = " - "+Error+" / "+ErrorDescription(Error)+" "+info;
  // nedestruktivni chyby
  switch(Error){
    case   4:Alert("Trade server is busy. Retrying.."+errTxt);
      Sleep(1000);
      return(true);
    case 137:Alert("Broker is busy. Retrying.."+errTxt);
      Sleep(1000);
      return(true);
    case 138:Alert("Requote. Retrying.."+errTxt);
      // pokud je potreba je nutne nacist znovu cenu
      return(true);
    case 146:Alert("Trading subsystem is busy. Retrying.."+errTxt);
      Sleep(500);
      return(true);
  }
  // kriticke chyby
  switch(Error){
    case   2:Alert("Common error."+errTxt);
      return(false);
    case  64:Alert("Account is blocked."+errTxt);
      return(false);
    case 133:Alert("Trading is prohibited"+errTxt);
      return(false);
    case 139:Alert("The order is blocked and is being processed"+errTxt);
      return(false);
    case 145:Alert("Modification is prohibited. ",
                   "The order is too close to market"+errTxt);
      return(false);
    default: Alert("ERROR"+errTxt); // neco jineho
  }
  return(false);
}

// serazeni poli sestupne podle druhe hodnoty (razeni podle nejvyssiho profitu)
// iTickets[] = {1,2,3} // pozor hodnoty nemohou zacinat o 0! min je 1
// dProfits[] = {100,30,50}
// result pak bude => {1,3,2} tj hodnoty 100,50,30
void sortArray(int &iTickets[], double &dProfits[], int &iResultTickets[]){
  // neni to prazdne
  if(ArraySize(iTickets) == 0) return(0);
  // promenne
  double sortedProfit[]; // target pro sortovani
  int x, xx, k = 0, v;
  bool cont = false;
  // rozmer pole
  ArrayResize(sortedProfit, ArraySize(iTickets));
  ArrayResize(iResultTickets, ArraySize(iTickets));
  ArrayCopy(sortedProfit, dProfits);
  // prvne seradime pole podle vydelku
  ArraySort(sortedProfit, WHOLE_ARRAY, 0, MODE_DESCEND);
  // preskladani
  for(x=0; x<ArraySize(sortedProfit); x++){
    for(xx=0; xx<ArraySize(iTickets); xx++){
      if(sortedProfit[x] == dProfits[xx]){
        // nemame to jiz pridane [stejna hodnota]?
        cont = false;
        for(v=0; v<ArraySize(iResultTickets); v++){
          if(iResultTickets[v] == iTickets[xx]){
            cont = true;
            break;
          }
        }
        if(cont == true) continue; // toto uz tam mame takze pokracujeme
        // pridame
        iResultTickets[k] = iTickets[xx];
        k++;
        break;
      }
    }
  }
}

// provede zavedeni obchodu, pokud se cokoliv nepovede vraci false
bool createTrades(int ibUpType, int ibDownType, int isUpType, int isDownType){
   // pro smycky
   int i,j;
   int pos = 0; // vypisovaci pozice
   int maxpos = 0;
   
   // cekajici obchody
   maxpos = ArraySize(aBuyPairs)*(ArraySize(aLotRanges)*2) + ArraySize(aSellPairs)*(ArraySize(aLotRanges)*2);
   // BUY
   for(i=0; i<ArraySize(aBuyPairs); i++){
     // nastavime jednotilive cekacky pro dany par
     for(j=0; j<ArraySize(aLotRanges); j++){
       pos++;
       Comment("CREATE BUY "+pos+"/"+maxpos+" "+aBuyPairs[i]+"/"+aLotSizes[j]+" - "+aLotRanges[j]);
       if(createWaitOrder(aBuyPairs[i], StrToDouble(aLotSizes[j]), ibUpType, StrToDouble(aLotRanges[j]), MagickNumber, MagickNumber) == false) return(false);
       pos++;
       Comment("CREATE BUY "+pos+"/"+maxpos+" "+aBuyPairs[i]+"/"+aLotSizes[j]+" - "+aLotRanges[j]);
       if(createWaitOrder(aBuyPairs[i], StrToDouble(aLotSizes[j]), ibDownType, StrToDouble(aLotRanges[j]), MagickNumber, MagickNumber) == false) return(false);
     }
   }
   // SELL
   for(i=0; i<ArraySize(aSellPairs); i++){
     Comment("CREATE SELL "+(i+1)+"/"+ArraySize(aSellPairs));
     // nastavime jednotilive cekacky pro dany par
     for(j=0; j<ArraySize(aLotRanges); j++){
       pos++;
       Comment("CREATE SELL "+pos+"/"+maxpos+" "+aSellPairs[i]+"/"+aLotSizes[j]+" - "+aLotRanges[j]);
       if(createWaitOrder(aSellPairs[i], StrToDouble(aLotSizes[j]), isUpType, StrToDouble(aLotRanges[j]), MagickNumber, MagickNumber) == false) return(false);
       pos++;
       Comment("CREATE SELL "+pos+"/"+maxpos+" "+aSellPairs[i]+"/"+aLotSizes[j]+" - "+aLotRanges[j]);
       if(createWaitOrder(aSellPairs[i], StrToDouble(aLotSizes[j]), isDownType, StrToDouble(aLotRanges[j]), MagickNumber, MagickNumber) == false) return(false);
     }
   }
   return(true);
}

// zavede cekajici pokyn
bool createWaitOrder(string symbol, double lot, int type, double range, string comment, int MN = 0){
  // zjistime aktualni cenu
  double actualPrice = 0.0;
  
  // provedeme pricteni range k aktualni cene
  switch(type){
   case OP_SELLLIMIT:
     actualPrice = MarketInfo(symbol, MODE_BID);
     actualPrice += convertPoint(symbol, range);
     break;
   case OP_BUYLIMIT:
     actualPrice = MarketInfo(symbol, MODE_ASK);
     actualPrice -= convertPoint(symbol, range);
     break;
   case OP_SELLSTOP:
     actualPrice = MarketInfo(symbol, MODE_BID);
     actualPrice -= convertPoint(symbol, range);
     break;
   case OP_BUYSTOP:
     actualPrice = MarketInfo(symbol, MODE_ASK);
     actualPrice += convertPoint(symbol, range);
     break;
   default:
     Alert("Nepovoleny typu obchodu!");
     return(false);
     break;
  }
  
  // zadani prikazu
  int ticket = OrderSend(symbol, type, lot, actualPrice, MaxSlippage, 0, 0, comment, MN);
  if(ticket > 0){  
    // vypiseme obchod na konzoli - nevypisujeme je to zbytecne pomale
    // if(OrderSelect(ticket, SELECT_BY_TICKET, MODE_TRADES)) OrderPrint(); 
  }else{
    Err = GetLastError();
    Alert(StringConcatenate("Error OrderSend? Symbol:", symbol, ", aPrice:", DoubleToStr(actualPrice, 5), ", Point:", DoubleToStr(convertPoint(symbol, range),5), ", Range: ", DoubleToStr(range,0), " - ", Err, " / ", ErrorDescription(Err)));
    return(false);
  }
  return(true);
}

//prevede body na cenu
double convertPoint(string symbol, double price){
 RefreshRates();
 double point = MarketInfo(symbol, MODE_POINT);  
 if(point == 0.00001)
   return(price * (point*10.0));
 if(point == 0.0001)
   return(price * (point));
 if(point == 0.001)
   return(price * (point*10.0));
 if(point == 0.01)
   return(price * (point));
}

// pokud je to mozne provede zamknuti, pokud se nepodarilo (nekdo jiz ma zamek) vratime false, jinak true
bool lock(){
  // testneme zda neexistuje locked
  if(GlobalVariableCheck(lockName) == true){
    return(false);
  }
  // zavadime lock
  GlobalVariableSet(lockName, internalMN);
  // test na to zda je lock zaveden a je nas
  if(GlobalVariableGet(lockName) == internalMN){
    // je to ok je to nas
    return(true);
  }
  return(false);
}

// provede odemknuti
void unlock(){
  if(GlobalVariableGet(lockName) == internalMN){
    GlobalVariableDel(lockName);
  }
}

/**
* create a positive integer for the use as a magic number.
*
* The function takes a string as argument and calculates
* an 31 bit hash value from it. The hash does certainly not 
* have the strength of a real cryptographic hash function 
* but it should be more than sufficient for generating a
* unique ID from a string and collissions should not occur.
*
* use it in your init() function like this: 
*    magic = makeMagicNumber(WindowExpertName() + Symbol() + Period());
*
* where name would be the name of your EA. Your EA will then
* get a unique magic number for each instrument and timeframe
* and this number will always be the same, whenever you put
* the same EA onto the same chart.
*
* Numbers generated during testing mode will differ from those
* numbers generated on a live chart.
*/
int makeMagicNumber(string key){
   int i, k;
   int h = 0;
   if (IsTesting()){
      key = "_" + key;
   }
   for (i=0; i<StringLen(key); i++){
      k = StringGetChar(key, i);
      h = h + k;
      h = bitRotate(h, 5); // rotate 5 bits
   }
   for (i=0; i<StringLen(key); i++){
      k = StringGetChar(key, i);
      h = h + k;
      // rotate depending on character value
      h = bitRotate(h, k & 0x0000000F);
   }
   // now we go backwards in our string
   for (i=StringLen(key); i>0; i--){   
      k = StringGetChar(key, i - 1);
      h = h + k;
      // rotate depending on the last 4 bits of h
      h = bitRotate(h, h & 0x0000000F); 
   }
   return(h & 0x7fffffff);
}

/**
* Rotate a 32 bit integer value bit-wise 
* the specified number of bits to the right.
* This function is needed for calculations
* in the hash function makeMacicNumber()
*/
int bitRotate(int value, int count){
   int i, tmp, mask;
   mask = (0x00000001 << count) - 1;
   tmp = value & mask;
   value = value >> count;
   value = value | (tmp << (32 - count));
   return(value);
}